use chronoptics_tof as tof;
use tof::Camera;

fn stream_frames() -> tof::Result<()> {
    let cameras = tof::discover_kea_cameras()?;

    println!("Detected the following cameras:");
    for camera in &cameras {
        println!("\t{}", camera.serial()?);
    }

    if cameras.is_empty() {
        println!("No kea cameras detected");
    } else {
        // Connect to first camera
        let msg = cameras.first().unwrap();
        println!("Connecting to camera {}", msg.serial()?);

        let processing_config = tof::ProcessingConfig::new_default()?;
        let mut cam = tof::KeaCamera::new(&processing_config, msg.serial()?)?;

        // Get ideal default processing for current camera config
        let cam_config = cam.get_camera_config()?;
        let default_processing_config = cam_config.default_processing()?;
        cam.set_process_config(&default_processing_config)?;

        let nr_streams = tof::select_streams(&cam, &[tof::FrameType::Z, tof::FrameType::Xyz])?;
        println!("Selected {nr_streams} streams");

        cam.start()?;

        for _ in 0..10 {
            let mut frames = cam.get_frames()?;

            for frame in &mut frames {
                print!(
                    "Frame type is {}",
                    tof::frame_type_to_string(frame.frame_type()?)?
                );

                match frame.frame_type()? {
                    tof::FrameType::Z => {
                        // Access const data
                        let data = frame.as_slice::<f32>()?;

                        println!(" len is {}", data.len());

                        // Access mut data
                        let _data = frame.as_mut_slice::<f32>()?;
                    }
                    tof::FrameType::Xyz => {
                        // Access const data
                        let data = frame.as_slice::<tof::XYZ>()?;

                        println!(" len is {}", data.len());

                        // Access mut data
                        let _data = frame.as_mut_slice::<tof::XYZ>()?;
                    }
                    _ => println!("Unhandled frame type"),
                }
            }
        }

        // This will be called automatically when cam runs out of scope
        cam.stop()?;
    }

    Ok(())
}

fn main() {
    let ret = stream_frames();

    match ret {
        Ok(_) => println!("Stream frames ran succesfully"),
        Err(err) => println!("Failed with {err}"),
    }
}
